package org.openjdk.gcbench.tests;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class Sequence implements Iterable<Object> {

    private final String descr;
    private final List<Object> vals;

    public static Sequence powersOfTen_Sub(int min, int max) {
        List<Object> vs = new ArrayList<>();
        for (int v = min; v <= max; v *= 10) {
            for (int t = 1; t <= 5; t++) {
                int n = v * t * 2;
                if (n <= max) {
                    vs.add(n);
                }
            }
        }
        return new Sequence("[" + min + ", " + max + "], power-of-ten steps, last step divided in 5 parts", vs);
    }

    public static Sequence powersOfTen(int min, int max) {
        List<Object> vs = new ArrayList<>();
        for (int v = min; v <= max; v *= 10) {
            vs.add(v);
        }
        return new Sequence("[" + min + ", " + max + "], power-of-ten steps", vs);
    }

    public static Sequence powersOfTwo(int min, int max) {
        List<Object> vs = new ArrayList<>();
        for (int v = min; v <= max; v *= 2) {
            vs.add(v);
        }
        return new Sequence("[" + min + ", " + max + "], power-of-two steps", vs);
    }

    public static Sequence powersOfTwo_WithMax(int min, int max) {
        List<Object> vs = new ArrayList<>();
        for (int v = min; v < max; v *= 2) {
            vs.add(v);
        }
        vs.add(max);
        return new Sequence("[" + min + ", " + max + "], power-of-two steps, with additional max", vs);
    }

    public static Sequence steps(int min, int max, int steps) {
        List<Object> vs = new ArrayList<>();
        int stepSize = (max - min) / steps;
        for (int v = min; v <= max; v += stepSize) {
            vs.add(v);
        }
        return new Sequence("[" + min + ", " + max + "], step size = " + stepSize, vs);
    }

    private Sequence(String descr, List<Object> vals) {
        this.descr = descr;
        this.vals = vals;
    }

    @Override
    public Iterator<Object> iterator() {
        return vals.iterator();
    }

    @Override
    public String toString() {
        return descr;
    }

    public static Sequence jvmModes() {
        return new Sequence("available JVM modes",
                Arrays.asList(
                        "-Xint",
                        "-XX:TieredStopAtLevel=1",
//                        "-XX:TieredStopAtLevel=2",
//                        "-XX:TieredStopAtLevel=3",
                        "-XX:TieredStopAtLevel=4"
                ));
    }

    public static Sequence javaType() {
        return new Sequence("Java types",
                Arrays.asList(
                        "none",
                        "boolean",
                        "byte",
                        "short",
                        "char",
                        "int",
                        "float",
                        "long",
                        "double",
                        "Object"
                ));
    }
}
